﻿// The code below has been extracted from the Open Rails project to
// support activity extraction in saved Installation Profiles. The code is
// distributed per the terms of the GNU General Public License. All copyright information
// has been left intact.

// The hope is if Open Rails chooses to change their implementation of Installation 
// Profiles, I can just copy and paste the new code here instead of rewriting my
// own code.   See my comment below - slipperman12.

// COPYRIGHT 2011, 2012, 2013 by the Open Rails project.
// 
// The original of this file was part of Open Rails - it has been amended for APKExtractor
// by G. Saunders (slipperman12 on UKTS MSTS Forum) April 2015 to include the Registry
// check for modern Open Rails installations.
// 
// Open Rails is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Open Rails is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Open Rails.  If not, see <http://www.gnu.org/licenses/>.

//  THIS IS A MODIFIED VERSION OF THIS MODULE TO WORK WITH APKExtractor V3.02  June 2015

using System.Collections.Generic;
using System.IO;
using MSTS;
using System;
using Microsoft.Win32;

namespace ORTS.Menu
{
    public class Folder : IComparable<Folder>
    {
        public readonly string Name;
        public readonly string Path;
      //  private static object path;

        public Folder(string name, string path)
        {
            Name = name;
            Path = path;
        }

        public override string ToString()
        {
            return Name;
        }

        public static string FolderDataFile
        {
            get
            {
               return Program.UserDataFolder + @"\folder.dat";
            }
        }

        public int CompareTo(Folder other)
        {
            return this.Name.CompareTo(other.Name);
        }

        public static List<Folder> GetFolders()
        {
            var folders = new List<Folder>();
           
            RegistryKey regkey = Registry.CurrentUser;                        // create base of Registry address
            regkey = regkey.OpenSubKey("Software\\OpenRails\\ORTS\\Folders"); // open the subkey [read-only]
            if (regkey == null)                                               // error handling - Reg key doesn't exist
            {                                                                
   
                if (File.Exists(FolderDataFile))
                {

                    try
                    {
                        using (var inf = new BinaryReader(File.Open(FolderDataFile, FileMode.Open)))
                        {
                            var count = inf.ReadInt32();
                            for (var i = 0; i < count; ++i)
                            {
                                var path = inf.ReadString();
                                var name = inf.ReadString();
                                folders.Add(new Folder(name, path));
                            }
                        }
                     }
                     catch { }
                }

            }
            else
            {
                string[] valnames = regkey.GetValueNames(); // retrieve the value names
                int i = 0;                                  // initialise index for arrays
                foreach (string name in valnames)           // for each value name retrieve its data
                {
                    string path = (string)regkey.GetValue(valnames[i]);
                    folders.Add(new Folder(name, path));    // add name and path into list
                    
                    i++;                                    // increment array index
                }
                regkey.Close();                             // close the Registry key

                folders.Sort();
            }
            if (folders.Count == 0)
            {
                try
                {
                    folders.Add(new Folder("Train Simulator", MSTSPath.Base()));
                }
                catch { }
            }

            return folders;
        }
        //  The following block of code is not required for APKExtractor
        //
         public static void SetFolders(List<Folder> folders)
         {
            using (var outf = new BinaryWriter(File.Open(FolderDataFile, FileMode.Create)))
            {
                outf.Write(folders.Count);
                foreach (var folder in folders)
                {
                   outf.Write(folder.Path);
                    outf.Write(folder.Name);
                }
            }
         }
        //  End of unwanted block
       
    }
}