﻿//    Copyright (C) 2013 Jay Herman

//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

//   THIS PROGRAM IS VERSION 3.07 February 2016 Modified by G. Saunders (slipperman12 on UKTS Forum)

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;


namespace APKExtractor
{
    class Program
    {
        public static string exception = "";

        static bool notAPKFile(string f)
           {
             return !(f.EndsWith(".apk", StringComparison.OrdinalIgnoreCase));
           }

        [STAThreadAttribute]

         static void Main(string[] args)
        {

            Application.EnableVisualStyles();

            ORTS.Menu.Program.Initialize();

           string progName = "APKExtractor3";
           string finalMsg = "The Installation has been Cancelled";
           string badMsg1 = "This program cannot be run by double-clicking its icon or exe file!\n\n";
           string badMsg1a = "Please use 'Drag 'n' Drop' to drop the apk file on to its Desktop icon or exe file, or set " + progName + ".exe";
           string badMsg1b = " as the default program to run .apk files.\n";
           string badMsg2 = "\n\nPress the OK button to quit.";
           string badMsg3 = "Invalid File type - only .apk files accepted\n";


           string tdbAnswer = "";

           if (args.Length < 1)
            {
                MessageBox.Show(badMsg1 + badMsg1a + badMsg1b + badMsg2, progName, MessageBoxButtons.OK, MessageBoxIcon.Error);

                return;
            }

            string fileName = args[0];

            string zipPath = fileName.Substring(0, fileName.Length - 3) + "zip";


            if (notAPKFile(fileName))      // if file type is NOT apk
            {
                MessageBox.Show(badMsg3 + badMsg2, progName, MessageBoxButtons.OK, MessageBoxIcon.Error);

                return;
            }
                         // if file type IS apk check if Zip selected

                _APKFile APKFile = new _APKFile(fileName);
                if (_APKFile.corruptFile)
                {
                    finalMsg = "UNABLE TO PROCESS apk FILE DUE TO INCOMPLETE or CORRUPTED DATA.\n";
                }
                else
                {
                    //  Valid apk file

                    string directory = null;


                    if (args.Length > 1)
                        directory = args[1];

                    if (directory == null)

                        FolderHelper.TryOpenRailsFolder(APKFile.FolderName, ref directory);


                    if (directory != null)
                    {
                        tdb m = new tdb();

                        string message = string.Format("Unpackage activity for {0} route to folder: {1}?", APKFile.RouteName, directory);
                        DialogResult result = MessageBox.Show(message, progName, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
                        switch (result)
                        {
                            case DialogResult.Yes:
                                {
                                    tdbAnswer = m.compareTDB(directory, APKFile.RouteID, APKFile.FolderName, _APKFile.tdbSer);

                                    if (tdbAnswer == "Continue")
                                    {
                                        if (APKFile.ExportToDirectory(directory))
                                        {
                                            finalMsg = "   ACTION COMPLETED\n\n apk file unpacked to : \n" + APKFile.RouteName + " route in " + directory + _APKFile.fileStats;
                                        }
                                        else
                                        {
                                            finalMsg = "UNABLE TO INSTALL ACTIVITY DUE TO : \n" + exception;
                                        }
                                        break;
                                    }
                                    else
                                    {
                                        if (tdbAnswer == "Cancel")
                                        {
                                            finalMsg = "   ACTION Cancelled by the User\n\n";
                                            break;
                                        }

                                    }
                                    break;
                                }
                            case DialogResult.No:
                                {
                                    using (FolderPicker folderPicker = new FolderPicker())
                                    {
                                        if (folderPicker.ShowDialog() == DialogResult.OK)
                                            if (folderPicker.FolderPicked == "Zip")
                                            {
                                                string zipName = fileName.Substring(0, fileName.Length - 3) + "zip";
                                                if (APKFile.ExportToZipFile(zipName))
                                                {
                                                    finalMsg = "   ACTION COMPLETED\n\n Zip file created :\n" + zipName;
                                                }
                                                else
                                                {
                                                    finalMsg = "UNABLE TO CREATE ZIP FILE.\n" + exception;
                                                }
                                            }
                                            else
                                            {
                                                //   APKFile.ExportToDirectory(folderPicker.FolderPicked);  // Original
                                                if (APKFile.ExportToDirectory(folderPicker.FolderPicked))
                                                {
                                                    finalMsg = "   ACTION COMPLETED\n\n apk file unpacked to :\n" + APKFile.RouteName + " route in " + folderPicker.FolderPicked + _APKFile.fileStats;
                                                }
                                                else
                                                {
                                                    finalMsg = "UNABLE TO INSTALL ACTIVITY DUE TO : \n" + exception;
                                                }
                                            }
                                    }
                                    break;
                                }
                            case DialogResult.Cancel:
                                {
                                    finalMsg = "    ACTION CANCELLED by the User\n\n";
                                    break;
                                }
                        }
                    }
                    else
                    {
                        string message = string.Format("The route {0} ({1}) could not be found in any of the selected locations. " +
                              "Would you like to choose a folder to extract the activity into, or create a zip file?", APKFile.RouteName, APKFile.FolderName);  // Original
                        DialogResult result = MessageBox.Show(message, progName, MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                        if (result == DialogResult.Yes)
                        {
                            using (FolderPicker folderPicker = new FolderPicker())
                            {
                                if (folderPicker.ShowDialog() == DialogResult.OK)

                                    if (folderPicker.FolderPicked == "Zip")
                                    {
                                        string zipName = fileName.Substring(0, fileName.Length - 3) + "zip";
                                        if (APKFile.ExportToZipFile(zipName))
                                        {
                                            finalMsg = "   ACTION COMPLETED\n\n Zip file created :\n" + zipName;
                                        }
                                        else
                                        {
                                            finalMsg = "UNABLE TO CREATE ZIP FILE.\n" + exception;
                                        }
                                    }
                                    else
                                    {
                                        //    APKFile.ExportToDirectory(folderPicker.FolderPicked);       //  Original

                                        if (APKFile.ExportToDirectory(folderPicker.FolderPicked))
                                        {
                                            finalMsg = "   ACTION COMPLETED\n\n apk file unpacked to :\n" + folderPicker.FolderPicked + _APKFile.fileStats;
                                        }
                                        else
                                        {
                                            finalMsg = "UNABLE TO INSTALL ACTIVITY DUE TO : \n" + exception;
                                        }
                                    }
                            }
                        }
                    }
                }
                    MessageBox.Show(finalMsg + "\nPress <OK> to close the program", progName, MessageBoxButtons.OK, MessageBoxIcon.Information);

      } // End of static void Main

   } // End of class Program

//*************************************************************************************************************************
    class tdb
    {

        public string compareTDB(string tsFolder, string rtID, string rtFolder, string tdbSer)
        {
            string tdbResult = "";
            string trkText = "FileName";                            // search value
            string tdbText = "Serial";                              // search value
            string buildTrk = "";
            string buildTdb = "";
           
            string[] trkFile = new string[3];


            string noTDB1 = "Unable to determine the route's Track Database Serial Number,\n";
            string noTDB2 = "probably due to invalid data in, or missing from, the trk and/or\ntdb files.\n\n ";
            string noTDB5 = "Do you wish to continue?";

            string noTRK1 = "No trk file is present in the Route folder, probably due to it\n";
            string noTRK2 = "being 'hidden' by Train Store or ROUTES ON/OFF.\n\n";
            string noTRK3 = "Press the OK button to cancel the installation of this activity,\n";
            string noTRK4 = "restore all the stored data then re-install the activity.\n";

          //  string routePath = tsFolder + "\\ROUTES\\" + rtID + "\\";  // Original
            string routePath = tsFolder + "\\ROUTES\\" + rtFolder + "\\";  // *** CORRECTION *** - v3.06
          //  string trkFilePath = routePath + "\\" + rtID + ".trk";    // Original
            string trkFilePath = routePath + rtID + ".trk";      // *** CORRECTION - v3.06 ***
            
            if (!File.Exists(trkFilePath))                                  //  If the specified trk file doesn't exist
            {
                    trkFile = Directory.GetFiles(routePath, "*.trk");       //  Check for any trk file in the folder
                    if (trkFile.Length > 0)                                 //  If there is at least one trk file
                    {
                        trkFilePath = trkFile[0];                           //  use the first one
                    }    
            }

            tdb n = new tdb();
          
            buildTrk = n.getTDBSerial(trkFilePath, trkText);

            if (buildTrk == "No TRK file")
            {
                DialogResult trkmsg1 = MessageBox.Show(noTRK1 + noTRK2 + noTRK3 + noTRK4, "APKExtractor3", MessageBoxButtons.OK, MessageBoxIcon.Information);
                switch (trkmsg1)
                {
                    case DialogResult.OK:
                        {
                            tdbResult = "Cancel";
                            break;
                        }
                }
            }
            else
            {
                if (buildTrk != "Invalid Data")                                                 //  If File name for trk file is valid
                {
                    string tdbFilePath = routePath + "\\" + buildTrk + ".tdb";
                    buildTdb = n.getTDBSerial(tdbFilePath, tdbText);

                    if (buildTdb == "No TDB file")
                    {
                        MessageBox.Show("No tdb file was found in the Route folder.\nPress OK to cancel this run.","APKExtractor3");
                        tdbResult = "Cancel";
                    }
                    else
                    {
                        if (buildTdb != "Invalid Data")                                           //  If file name for tdb file is valid
                        {
                            if (buildTdb != tdbSer)                                             //  If track database serial numbers don't match
                            {
                                string tdbmessage = string.Format("The installed track database for this route is version {1}; the activity was built on version {0}.  Do you wish to Continue?", tdbSer, buildTdb);
                                DialogResult tdbmsg1 = MessageBox.Show(tdbmessage, "APKExtractor3", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                                switch (tdbmsg1)
                                {
                                    case DialogResult.No:                                      // If route not accepted
                                        {
                                            tdbResult = "Cancel";
                                            break;
                                        }
                                    case DialogResult.Yes:                                    // If route is accepted
                                        {
                                            tdbResult = "Continue";
                                            break;
                                        }
                                }
                            }
                            else
                            {
                                tdbResult = "Continue";
                            }
                        }
                    }
                }  // End of check for valid trk file
            }       // End of trk file present

            if ((buildTrk == "Invalid Data") || (buildTdb == "Invalid Data"))         // If either or both of the trk/tdb file names is/are invalid
            {
                DialogResult tdberrans = MessageBox.Show(noTDB1 + noTDB2 + noTDB5, "APKExtractor3", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                switch (tdberrans)
                {
                    case DialogResult.Yes:
                        {
                            tdbResult = "Continue";
                            break;
                        }
                    case DialogResult.No:
                        {
                            tdbResult = "Cancel";
                            break;
                        }
                }
            }               //  End of invalid data message box

            return tdbResult;
        }  // End of compare TDB

        //*******************************************************************************************************************************************************************
        public string getTDBSerial(string fName, string seekText)
        {
            int firstPosn, lastPosn;
            string dQuot = "\"";

            string fLine = "";
            string lineFound = "";
            string buildVal = "";
            string fileParam;
            string seekText2;

            if (seekText == "FileName")
            {
                fileParam = "TRK";
                seekText2 = "Filename";
            }
            else
            {
                fileParam = "TDB";
                seekText2 = "serial";
            }

            if (File.Exists(fName))
            {
                System.IO.StreamReader file = new System.IO.StreamReader(fName);
                while ((fLine = file.ReadLine()) != null)
                {

                    if (fLine.Contains(seekText) || fLine.Contains(seekText2))
                    {
                        lineFound = fLine;
                        break;
                    }
                }
                file.Close();

                int openBracket = lineFound.IndexOf("(");
                int closeBracket = lineFound.IndexOf(")");

                if ((openBracket == -1) || (closeBracket == -1))
                {
                    return "Invalid Data";
                }
                else
                {
                    firstPosn = openBracket + 1;
                    lastPosn = closeBracket - 1;
                    if (lineFound.Substring(firstPosn, 1) == " ")
                    { firstPosn++; }

                    if (lineFound.Substring(lastPosn, 1) == " ")
                    { lastPosn--; }

                    if (fileParam == "TRK")
                    {
                        if (lineFound.Substring(firstPosn, 1) == dQuot)
                        { firstPosn++; }

                        if (lineFound.Substring(lastPosn, 1) == dQuot)
                        { lastPosn--; }
                    }
                    for (int i = firstPosn; !(i > lastPosn); i++)
                    {
                        buildVal = buildVal + lineFound.Substring(i, 1);
                    }
                    return buildVal;
                } // end of else (brackets test)
            }  // End of File.Exists check
            else
            {
                return "No " + fileParam + " file";
            }

        }  // End of getTDBSerial

    }  // End of class tdb
//*******************************************************************************************************************************

}  // End of namespace APKExtractor