﻿//    Copyright (C) 2013 Jay Herman

//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

//     THIS PROGRAM IS VERSION 3.07 February 2016 G. Saunders (slipperman12 @ UKTS)

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using ICSharpCode.SharpZipLib.GZip;
using ICSharpCode.SharpZipLib.Zip;
using System.IO.Compression;
using System.Threading.Tasks;

namespace APKExtractor
{
    class sortexistingFiles // : IComparable<sortexistingFiles>
    {
        
        public string existName { get; set; }
        public int existIndex { get; set; }

    }
    
    class _APKFile
    {
        public static int fctr = 0;
        public static int byteTest = 0;
        private string routeName = string.Empty;
        private string folderName = string.Empty;
        private string routeID = string.Empty;
        public static int filesToBeCreated;
        public static string tdbSer;
        public static List<string> existName = new List<string>();
        public static List<int> existIndex = new List<int>();
        public static string fileStats = "";
        public static bool corruptFile = false;

        private List<_APKContents> fileContents = new List<_APKContents>();

        public string RouteName
        {
            get { return routeName; }
        }

        public string FolderName
        {
            get { return folderName; }
        }

        public string RouteID
        {
            get { return routeID;  }
        }

        public List<string> ExistName
        {
            get { return existName; }
        }


        public List<_APKContents> FileContents
        {
            get { return fileContents; }
        }

        public _APKFile(String fileName)
        {
            FileStream cFile = new FileStream(fileName, FileMode.Open, FileAccess.Read);

            GZipStream uFile = new GZipStream(cFile, CompressionMode.Decompress);

            _APKReader aReader = new _APKReader(uFile);
            List<Byte[]> fields = new List<Byte[]>();


            foreach (Byte[] field in aReader.ReadFields())
            { 
                fields.Add(field);
                if (corruptFile) return;
            }
            if (!corruptFile)
            {
                //fields[0] = Length of Route Name
                //fields[1] = Route Name
                routeName = Encoding.Unicode.GetString(fields[1]);

                //fields[2] = Length of Folder Name
                //fields[3] = Folder Name
                folderName = Encoding.Unicode.GetString(fields[3]);

                //fields[4] = Length of Route ID
                //fields[5] = Route ID
                routeID = Encoding.Unicode.GetString(fields[5]);

                //fields[6] = Track Database Serial Number

                //fields[7] = Count of files to be created

                for (int index = 8; index <= fields.Count - 2; index += 4)
                {
                    //fields[8,12,...] = Length of the following file
                    //fields[9,13,...] = Length of File Name
                    //fields[10,14,...] = File Name
                    //fields[11,15,...] = File
                    fileContents.Add(new _APKContents(fields[index + 2], fields[index + 3], (index >= fields.Count - 4)));
                }
            }
            else
            {
                return;
            }
        }

        public bool ExportToZipFile(string fileName)
        {
            string excName = "";
            bool validRun = true;
            FileStream fsOut = File.Create(fileName);
            ZipOutputStream zipStream = new ZipOutputStream(fsOut);
            zipStream.SetLevel(9);
            foreach (_APKContents contents in fileContents)
            {
                string entryName = contents.FileName;
                if (IsLegalFileName(entryName))
                {
                    entryName = ZipEntry.CleanName(entryName);
                    ZipEntry newEntry = new ZipEntry(entryName);
                    newEntry.Size = contents.Contents.Length;
                    zipStream.PutNextEntry(newEntry);
                    zipStream.Write(contents.Contents.ToArray(), 0, (int)contents.Contents.Length);
                }
                else
                {
                    validRun = false;
                    excName = entryName;
                    zipStream.Close();
                    if (File.Exists(fileName))
                    {
                        File.Delete(fileName);
                    }
                    break;
                }
            }  // End of foreach
            if (validRun)
            {
                zipStream.Finish();
                zipStream.Close();
                return true;
            }
            else
            {
                Program.exception = "File name contains illegal characters.";
                return false;
            }
        }

        public static bool IsLegalFileName(string name)
        {
            // Check that the path doesn't contain illegal characters
            bool ret = true;
            try
            {
                string checkName = Path.GetFileName(name);
                string checkFolder = Path.GetDirectoryName(name);
            }
            catch (ArgumentException)
            {
               ret = false;
            }
            return ret;
        }


        List<sortexistingFiles> list2 = new List<sortexistingFiles>();
        public bool ExportToDirectory(string directory)
        {
            List<string> fileName = new List<string>();
            List<string> fileFlag = new List<string>();
            List<int> fileIndex = new List<int>();

            int fileCount = 0;
            int fileExists = 0;
            string filePath2 = "";
            try
            {
                foreach (_APKContents contents in fileContents)
                {
                    int numFiles = fileContents.Count;
                    filePath2 = Path.Combine(directory, contents.FileName);
                    fileName.Add(Path.GetFileName(filePath2));   //  store file name
                    fileIndex.Add(fileCount);                   //  store main index number = position in apk file
                
                    if (File.Exists(filePath2))                  //  if the file already exists -
                    {
                        fileFlag.Add("X");                          //  set the flag to X (eXisting)
                        list2.Add(new sortexistingFiles() { existName = fileName[fileCount], existIndex = fileCount });
                        fileExists++;                               //  increase the existing files counter by i
                    }
                    else                                        //  if the file doesn't already exist
                    {
                       fileFlag.Add("W");                           //  set the flag to W (Write)
                    }
                    fileCount++;                                //  Increase the file counter by 1
                }  // end of foreach
            }
            catch (Exception ex)
            {
                Program.exception = ex.ToString() + "\n Path = " + filePath2;
                fileExists = 0;
                return false;
            }

            if (fileExists > 0)                              //  if there are any existing files
            {

           //  routine to sort filenames into .act, .pat, .srv, .trf and .con sequence to be inserted here

                foreach (var fname in list2)                        //  Temporarily change .con extension to .xon
                {
                    if (fname.existName.Substring(fname.existName.Length - 4) == ".con")
                    {
                        fname.existName = fname.existName.Replace(".con", ".xon");
                    }
                    fname.existName = fname.existName.Substring(fname.existName.Length - 3) + fname.existName;
                }

                list2.Sort(delegate(sortexistingFiles x, sortexistingFiles y) {return x.existName.CompareTo(y.existName); });

                foreach (var fname in list2)                        //  Revert .xon back to .con
                {
                    if (fname.existName.Substring(fname.existName.Length - 4) == ".xon")
                    {
                        fname.existName = fname.existName.Replace(".xon", ".con");
                    }
                    fname.existName = fname.existName.Substring(3,fname.existName.Length - 3);
                }
                foreach (var element in list2)
                {
                    existName.Add(element.existName);
                    existIndex.Add(element.existIndex);
                }
                using (OverWrite overwrite = new OverWrite())
                {
                    overwrite.ShowDialog();
                    List<int> toOverwrite = new List<int>(OverWrite.selected);   // list of screen index numbers of files selected for overwriting
                    for (int i = 0; i < toOverwrite.Count; i++)                  // for each index number selected for overwriting
                    {
                        fileFlag[existIndex[toOverwrite[i]]] = "O";           //  set flag to letter O (overwrite)
                    }
                }
            }    //  end of existing files
                 //  Output only those files with flag = "W" or "O"
            int jj = 0;
            int filesOverwritten = 0;
            int filesIgnored = 0;
            int filesWritten = 0;
            try
            {
                foreach (_APKContents contents in fileContents)
                {
                    string filePath3 = Path.Combine(directory, contents.FileName);
                    if ((fileFlag[jj] == "W") || (fileFlag[jj] == "O"))
                    {
                        string subDirectory = Path.GetDirectoryName(filePath3);
                        if (!Directory.Exists(subDirectory)) Directory.CreateDirectory(subDirectory);
                        using (FileStream fsOut = File.Create(filePath3))
                        {
                            fsOut.Write(contents.Contents.ToArray(), 0, (int)contents.Contents.Length);
                            fsOut.Close();
                        }
                        if (fileFlag[jj] == "O")
                        {
                            filesOverwritten++;
                        }
                        else
                        {
                            filesWritten++;
                        }
                    }
                    else
                    {
                        filesIgnored++;
                    }
                    jj++;
                }     // end of all output

                string fileStats1 = "\n\nNumber of files to be output : " + fileCount;
                string fileStats2 = "\nNumber of new files written  : " + filesWritten;
                string fileStats3 = "\nNumber of files overwritten  : " + filesOverwritten;
                string fileStats4 = "\nNumber of files skipped         : " + filesIgnored;
                string fileStats5 = "\nActual number of files output : " + (filesWritten + filesOverwritten);
                fileStats = fileStats1 + fileStats2 + fileStats3 + fileStats4 + fileStats5;

                return true;
            }
            catch (Exception ex)
            {
                Program.exception = ex.ToString();
                return false;
            }
        }
    }

    class _APKContents
    {
        private string fileName;
        private MemoryStream contents;

        public _APKContents(byte[] nameBuffer, byte[] contentsBuffer, bool lastFile)
        {
            fileName = Encoding.Unicode.GetString(nameBuffer);

            int bufferLength = contentsBuffer.Length;

            contents = new MemoryStream(contentsBuffer, 0, bufferLength);

         }

        public string FileName
        {
            get { return fileName; }
        }

        public MemoryStream Contents
        {
            get { return contents; }
        }
    }

    class _APKReader : BinaryReader
    {
        public static List<Byte> bytes = new List<Byte>();


        public _APKReader(Stream input)
            : base(input)
        { }

        public IEnumerable<byte[]> ReadFields()
        // This routine replaces the original. G. Saunders (UKTS slipperman12) April-May 2015
        {
            Byte[] b = new Byte[2];
            Byte[] b4 = new Byte[4];
            long bytesRead;
            String hexString, hex;
            int numFiles = 0, cond = 2;         // Conditional counter (cond) set to two as the first 6 fields are created in pairs
            for (int q = 0; q < 8; q = q + cond)   // First 8 fields extraction
            {
                hexString = null;
                bytesRead = this.Read(b4, 0, 4);
                                                    //  Fields 0, 2, 4, 6 & 7
                for (int hx = 0; hx < 4; hx++ )
                {
                    hex = b4[hx].ToString("X").PadLeft(2, '0');
                    hexString = hex + hexString;
                }

                int value = Convert.ToInt32(hexString, 16);
                bytes.AddRange(b4);
                yield return bytes.ToArray();
                bytes.Clear();

                if (q < 6)
                {                                    //  Fields 1, 3 & 5
                    hexString = null;
                    for (int k = 0; k < (value); k++)  //  Read string name 2 bytes at a time
                    {
                        bytesRead = this.Read(b, 0, 2);
                        if (b[0] == 0 && b[1] == 0)     //  Ignore two dummy (blank) bytes at end of name
                        {
                            yield return bytes.ToArray();
                            bytes.Clear();
                        }
                        bytes.AddRange(b);
                    }
                }
                else
                {
                    cond = 1;                       // Reset conditional counter to 1 to access data for fields 6 & 7
                    if (q == 7)                     // The 8th field (Field 7) contains a count of the number of files to be created
                    { numFiles = value;
                    _APKFile.filesToBeCreated = value;
                    }           //  Save it for use later

                    if (q == 6)                                     //  Track Database Serial Number
                    { _APKFile.tdbSer = Convert.ToString(value);

                    }   //  Converted to a string for comparison with apk file value
                }
                
            }  // End of first 8 fields extraction 'for' loop

            int numFilesCount = 0;
            int p = 8, lenFile = 0, lenName = 0;

            while (numFilesCount < numFiles)
            {
                for (int x = 0; x < 2; x++)
                {                                       //  Read Fields 8 & 9 and their equivalents
                    hexString = null;
                    bytesRead = this.Read(b4, 0, 4);
                    for (int hx = 0; hx < 4; hx++)
                    {
                        hex = b4[hx].ToString("X").PadLeft(2, '0');
                        hexString = hex + hexString;
                    }
                    int value = Convert.ToInt32(hexString, 16);
                    bytes.AddRange(b4);
                    yield return bytes.ToArray();
                    bytes.Clear();

                    if (x == 0)
                    {
                        lenFile = value;                //  Length of the following file
                    }
                    else
                    {
                        lenName = value;                //  Length of the following file's name
                    }
                    p++;
                }   //  End of reading Fields 8 & 9 ( + equivalents)


                for (int k = 0; k < (lenName); k++)
                {
                    bytesRead = this.Read(b, 0, 2);
                    if (b[0] == 0 && b[1] == 0)                 //  Ignore two blank bytes at the end of the name
                    {
                        yield return bytes.ToArray();
                        bytes.Clear();
                    }
                    else
                    {
                        bytes.AddRange(b);
                    }
                   
                }               // End of Field 10.... extraction
                p++;
                
                for (int m = 0; m < (lenFile/2); m++)
                {
                    bytesRead = this.Read(b, 0, 2);
                    if (bytesRead <= 0)
                    {
                        _APKFile.corruptFile = true;
                        yield break;
                    }
                    else
                    {
                        bytes.AddRange(b);
                    }
 
                }
                yield return bytes.ToArray();
                bytes.Clear();
                p++;
                numFilesCount++;            // End of Field 11 .... extraction - Files count increased by 1


            }       //  End of unpacked apk file reading

        }   // End of InEnumerable  ReadFields

    }  // End of class _APKReader : Binary Reader

 }  // End of namespace APKExtractor